/*---[ firestarter.c ]------------------------------------------------
 * Copyright (C) 2000-2002 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The main application file
 *--------------------------------------------------------------------*/

#include <config.h>
#include <gnome.h>
#include <sys/stat.h>
#include <stdio.h>
#include <netdb.h>
#include <sys/socket.h>
#include <errno.h>
#include <popt.h>

#include "globals.h"
#include "firestarter.h"
#include "gui.h"
#include "menus.h"
#include "util.h"
#include "logread.h"
#include "druid.h"

FirestarterApp Firestarter;

static gint save_session (GnomeClient       *client,
                          gint               phase,
                          GnomeSaveStyle     save_style,
                          gint               is_shutdown,
                          GnomeInteractStyle interact_style,
                          gint               is_fast,
                          gpointer           client_data);

static void session_die (GnomeClient        *client,
                         gpointer            client_data);

static gint appbar_update (gchar *state);	  

/* [ stop_firewall ]
 * Flushes, zeroes and sets all policies to accept
 */
void
stop_firewall (void)
{
	/* gchar *soundfile; */
	gchar *sbin;
	gchar *command = NULL;

	if (root_alert ())
		return;

	if (gnome_config_get_bool ("/firestarter/Druid/locatesbins=FALSE"))
		sbin = g_strdup ("`which iptables`");
	else
		sbin = g_strdup ("/sbin/iptables");

	command = g_strconcat (sbin, " -F;",
	                       sbin, " -X;",
	                       sbin, " -Z;",
	                       sbin, " -P INPUT ACCEPT;",
	                       sbin, " -P FORWARD ACCEPT;",
	                       sbin, " -P OUTPUT ACCEPT;", NULL);
	system (command);
	g_print (_("Firewall stopped, network traffic is now flowing freely\n"));
	appbar_update ("stop");

	g_free (sbin);
	g_free (command);
}

/* [ restart_firewall ]
 * Executes the firewall script
 */
void
restart_firewall (void)
{
	gchar *command = g_strjoin (NULL, "/bin/sh ",
	                            FIRESTARTER_RULES_DIR "/firestarter/firewall.sh",
	                            NULL);

	if (root_alert ())
		return;

	system (command);
	g_print (_("Firewall script restarted\n"));
	appbar_update ("start");

	g_free (command);
}

/* [ halt_traffic ]
 * Flushes and sets all policies to deny
 */
void
halt_traffic (void)
{
	gchar *sbin;
	gchar *command;

	if (root_alert ())
		return;

	if (gnome_config_get_bool ("/firestarter/Druid/locatesbins=TRUE"))
		sbin = g_strdup ("`which iptables`");
	else
		sbin = g_strdup ("/sbin/iptables");

	command = g_strconcat (sbin, " -P INPUT DROP;",
	                       sbin, " -P FORWARD DROP;",
	                       sbin, " -P OUTPUT DROP;",
	                       sbin, " -F;",
	                       sbin, " -X;",
	                       sbin, " -Z;", NULL);
	system (command);
	g_print (_("All traffic halted\n"));
	appbar_update ("halt");

	g_free (sbin);
	g_free (command);
}

/* [ exit_firestarter ]
 * Quit firestater
 */
void
exit_firestarter (void)
{
	if (gnome_config_get_bool ("/firestarter/Program/shutdown=FALSE"))
	stop_firewall ();
	gtk_main_quit ();
}

/* [ save_session ]
 * Saves the current session for later revival
 */
static gint
save_session (GnomeClient       *client,
              gint               phase,
              GnomeSaveStyle     ave_style,
              gint               is_shutdown,
              GnomeInteractStyle interact_style,
              gint               is_fast,
              gpointer           client_data)
{
	gchar **argv;
	guint argc;

	argv = g_new0 (gchar*, 4);
	argc = 1;

	argv[0] = client_data;
	gnome_client_set_clone_command (client, argc, argv);
	gnome_client_set_restart_command (client, argc, argv);
	return TRUE;
}

/* [ session_die ]
 * Gracefully end the session
 */
static void
session_die (GnomeClient *client, gpointer client_data)
{
	exit_firestarter ();
}

/* [ appbar_update ]
 * Updates the application status bar with the state of the firewall,
 * returns the new state
 */
static gint
appbar_update (gchar *state)
{
	static int currentstate;

	if (state == "stop") {
		if (!NO_APPLET) gnome_appbar_set_default (GNOME_APPBAR (appbar),
			_("Firewall stopped"));
		currentstate = 1;
	}
	else if (state == "start") {
		if (!NO_APPLET) gnome_appbar_set_default (GNOME_APPBAR (appbar),
			_("Firewall running"));
		currentstate = 2;
	}
	else if (state == "halt") {
		if (!NO_APPLET) gnome_appbar_set_default (GNOME_APPBAR (appbar),
			_("All traffic halted"));
		currentstate = 3;
	}

	return currentstate;
}

/* [ main ]
 * The main function, this is where it all begins and ends
 */
int
main (int argc, char* argv[])
{
	GnomeProgram *firestarter;
	GnomeClient *client;
	
	/* command line stuff */
	poptContext ctx;
	gint opt_up, opt_down, opt_stop;
	
	const struct poptOption options[] =
	{
		{"noapplet", 'a', POPT_ARG_NONE, &NO_APPLET, 0, _("Startup without panel applet"), NULL},
		{"start", 'u', POPT_ARG_NONE, &opt_up, 0, _("Start the firewall"), NULL},
		{"stop", 'd', POPT_ARG_NONE, &opt_down, 0, _("Stop the firewall"), NULL},
		{"halt", 'h', POPT_ARG_NONE, &opt_stop, 0, _("Halt all network traffic"), NULL},
		{NULL, 0, 0, NULL, 0, NULL, NULL}
	};
	
	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
	textdomain (GETTEXT_PACKAGE);

	opt_up = opt_down = opt_stop = 0;
	ctx = poptGetContext (NULL, argc, argv, options, 0);
	poptGetNextOpt( ctx );
	poptResetContext (ctx);

	if (opt_up || opt_down || opt_stop)
	{
		CONSOLE = TRUE;
		NO_APPLET = TRUE;

		if (opt_up)
			restart_firewall ();
		else if (opt_down)
			stop_firewall ();
		else if (opt_stop)
			halt_traffic ();
	
		return 0;
	}

	firestarter = gnome_init_with_popt_table ("firestarter", VERSION, argc, argv,
		options, 0, &ctx);

/* Set up the session managment */
	client = gnome_master_client ();
	g_signal_connect (G_OBJECT (client), "save_yourself",
			  G_CALLBACK (save_session), argv[0]);
	g_signal_connect (G_OBJECT (client), "die",
			  G_CALLBACK (session_die), NULL);

/* Finalization stage */

	/* Creating the dir for storing scripts if it doesn't exist */
	mkdir (FIRESTARTER_RULES_DIR "/firestarter", 00700);

	/* See that the scripts are for the correct program version */
//	check_version ();

	/* Creating the GUI */
	gui_construct ();

	/* Initialize the system log file polling function */
	open_logfile (gnome_config_get_string (
		"/firestarter/Files/logfile=/var/log/messages"));

	/* If we're starting Firestarter for the first time, proceed to wizard */
	if (gnome_config_get_bool ("/firestarter/Druid/firsttime=TRUE"))
		rundruid ();
	else {
		if (gnome_config_get_bool ("/firestarter/Program/startup=TRUE"))
			restart_firewall ();
		else
			appbar_update ("stop");
	}

	gtk_main ();

	return 0;
}
