/*---[ menus.c ]------------------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu and toolbar related functions
 *--------------------------------------------------------------------*/

#include "menus.h"
#include "firestarter.h"
#include "druid.h"
#include "savelog.h"
#include "preferences.h"
#include "modrules.h"
#include "gui.h"
#include "hitview.h"

#include "xpm/pixbufs-images.h"
#include "xpm/wizard.xpm"
#include "xpm/clear.xpm"
#include "xpm/refresh.xpm"
#include "xpm/save.xpm"
#include "xpm/start.xpm"
#include "xpm/stop.xpm"

enum {
	TOOLBAR_WIZARD,
	TOOLBAR_SEP1,
	TOOLBAR_CLEAR,
	TOOLBAR_RELOAD,
	TOOLBAR_SAVE,
	TOOLBAR_SEP2,
	TOOLBAR_START,
	TOOLBAR_STOP,
};

static void goto_homepage (void);
static void goto_manual (void);
static GtkWidget *create_hits_popup (void);
static GtkWidget *create_rules_popup (void);

GnomeUIInfo toolbar_data[] = {

	GNOMEUIINFO_SEPARATOR,

	{ GNOME_APP_UI_ITEM, 
	  N_("Reload"), N_("Reload firewall hit list"), 
	  hitview_reload, NULL, NULL,
	  GNOME_APP_PIXMAP_STOCK, GTK_STOCK_REVERT_TO_SAVED,
	  0, 0, NULL },
	
	{ GNOME_APP_UI_ITEM, 
	  N_("Save"), N_("Save firewall hit list to file"), 
	  create_savelog_filesel, NULL, NULL,
	  GNOME_APP_PIXMAP_STOCK, GTK_STOCK_SAVE_AS,
	  0, 0, NULL },

	GNOMEUIINFO_SEPARATOR,

	GNOMEUIINFO_END
};

static GnomeUIInfo firewall_menu [] = {
	GNOMEUIINFO_ITEM (N_("_Run firewall wizard"), NULL, rundruid, wizard_xpm),
	GNOMEUIINFO_ITEM (N_("Start _firewall"), NULL, restart_firewall, start_xpm),
	GNOMEUIINFO_ITEM (N_("_Stop firewall"), NULL, stop_firewall, stop_xpm),
	GNOMEUIINFO_ITEM (N_("_Halt all network traffic"), NULL, halt_traffic, stop_xpm),
	GNOMEUIINFO_MENU_PREFERENCES_ITEM (preferences_cb, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM (exit_firestarter, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo hit_list_menu [] = {
	GNOMEUIINFO_ITEM (N_("_Clear firewall hit list"), NULL, hitview_clear, clear_xpm),
	GNOMEUIINFO_ITEM (N_("R_eload entire firewall hit list"), NULL, hitview_reload, refresh_xpm),
	GNOMEUIINFO_ITEM (N_("_Save firewall hit list to file"), NULL, create_savelog_filesel, save_xpm),
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu [] = {
	GNOMEUIINFO_ITEM_STOCK (N_("Online _Manual"), NULL, goto_manual, GNOME_STOCK_MENU_SEARCH),
	GNOMEUIINFO_ITEM_STOCK (N_("Firestarter _Homepage"), NULL, goto_homepage, GNOME_STOCK_MENU_HOME),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_ABOUT_ITEM (show_about, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menu [] = {
	GNOMEUIINFO_SUBTREE (N_("_Firewall"), firewall_menu),
	GNOMEUIINFO_SUBTREE (N_("_Hit list"), hit_list_menu),
	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
	GNOMEUIINFO_END
};

static GtkItemFactory *hits_popup_factory = NULL;
static GtkWidget *hits_popup_menu = NULL;
static int hits_popup_entries_qty = 18;

static GtkItemFactoryEntry hits_popup_entries[] =
{
	{N_("/Block host"), NULL, block_host, 0, NULL},
	{N_("/Trust host"), NULL, trust_host, 0, NULL},
	{N_("/Open port"), NULL, open_port, 0, NULL},
	{N_("/Open port for this host only"), NULL, open_port_for_machine, 0, NULL},
	{N_("/Block and stop logging port"), NULL, block_port, 0, NULL},
	{N_("/Lookup hostname"), NULL, lookup_selected_hit, 0, NULL},
	{   "/---", NULL, NULL, 0, "<Separator>"},
	{N_("/View columns"), NULL, 0, 0, "<Branch>"},
	{N_("/View columns/Time"), NULL, hitview_toggle_column_visibility, 0, "<CheckItem>"},
	{N_("/View columns/In"), NULL, hitview_toggle_column_visibility, 1, "<CheckItem>"},
	{N_("/View columns/Out"), NULL, hitview_toggle_column_visibility, 2, "<CheckItem>"},
	{N_("/View columns/Port"), NULL, hitview_toggle_column_visibility, 3, "<CheckItem>"},
	{N_("/View columns/Source"), NULL, hitview_toggle_column_visibility, 4, "<CheckItem>"},
	{N_("/View columns/Destination"), NULL, hitview_toggle_column_visibility, 5, "<CheckItem>"},
	{N_("/View columns/Length"), NULL, hitview_toggle_column_visibility, 6, "<CheckItem>"},
	{N_("/View columns/ToS"), NULL, hitview_toggle_column_visibility, 7, "<CheckItem>"},
	{N_("/View columns/Protocol"), NULL, hitview_toggle_column_visibility, 8, "<CheckItem>"},
	{N_("/View columns/Service"), NULL, hitview_toggle_column_visibility, 9, "<CheckItem>"}
};

static GtkItemFactory *rules_popup_factory = NULL;
static GtkWidget *rules_popup_menu = NULL;
static int rules_popup_entries_qty = 3;

static GtkItemFactoryEntry rules_popup_entries[] =
{
	{N_("/Remove rule"), NULL, remove_rule, 0, NULL},
	{N_("/Edit rule"), NULL, edit_selected_rule, 0, NULL},
	{N_("/New rule"), NULL, create_new_rule, 0, NULL}
};

static GtkWidget *
create_icon (const guint8 rgba_data [])
{
	GtkWidget *image;
	GdkPixbuf *pixbuf;
	
	pixbuf = gdk_pixbuf_new_from_inline (-1, rgba_data, FALSE, NULL);

	if (pixbuf == NULL)
		return NULL;

	image = gtk_image_new_from_pixbuf (pixbuf);
	g_object_unref (pixbuf);

	return image;
}

/* [ install_menus_and_toolbar ]
 * Install the menus and the toolbar in *window
 */
void
install_menus_and_toolbar (GtkWidget *window)
{
	GtkWidget *toolbar;

	gnome_app_create_menus_with_data (GNOME_APP (window), menu, window);

	toolbar = gtk_toolbar_new ();
	gnome_app_fill_toolbar_with_data (GTK_TOOLBAR (toolbar), 
	                                  toolbar_data, 
	                                  (GtkAccelGroup*) NULL,
	                                  window);

	gtk_toolbar_insert_element (GTK_TOOLBAR (toolbar),
				    GTK_TOOLBAR_CHILD_BUTTON,
				    NULL, 
				    _("Wizard"), 
				    _("Run firewall wizard"),
				    NULL, 
				    create_icon (wizard_icon),
				    GTK_SIGNAL_FUNC (rundruid), 
				    window,
				    TOOLBAR_WIZARD);

	gtk_toolbar_insert_element (GTK_TOOLBAR (toolbar),
				    GTK_TOOLBAR_CHILD_BUTTON,
				    NULL, 
				    _("Clear"), 
				    _("Clear firewall hit list"),
				    NULL, 
				    create_icon (clear_icon),
				    GTK_SIGNAL_FUNC (hitview_clear), 
				    window,
				    TOOLBAR_CLEAR);

	gtk_toolbar_insert_element (GTK_TOOLBAR (toolbar),
				    GTK_TOOLBAR_CHILD_BUTTON,
				    NULL, 
				    _("Restart"), 
				    _("Restart firewall"),
				    NULL, 
				    create_icon (start_icon),
				    GTK_SIGNAL_FUNC (restart_firewall), 
				    window,
				    TOOLBAR_START);

	gtk_toolbar_insert_element (GTK_TOOLBAR (toolbar),
				    GTK_TOOLBAR_CHILD_BUTTON,
				    NULL, 
				    _("Stop"), 
				    _("Stop firewall"),
				    NULL, 
				    create_icon (stop_icon),
				    GTK_SIGNAL_FUNC (stop_firewall), 
				    window,
				    TOOLBAR_STOP);

	gnome_app_set_toolbar (GNOME_APP (window), GTK_TOOLBAR (toolbar));

	appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
	gnome_app_set_statusbar (GNOME_APP (window), appbar);
	
	hits_popup_menu = create_hits_popup ();
	rules_popup_menu = create_rules_popup ();
}

/* [ hitview_popup_menu ]
 * Pops up a menu and sets the sensitivity of the menu items
 */
void
hitview_popup_menu (GdkEventButton *bevent)
{
	gtk_menu_popup (GTK_MENU (hits_popup_menu),
	                          NULL, NULL, NULL, NULL,
	                          bevent->button, bevent->time);
}

/* [ ruleview_popup_menu ]
 * Pops up a menu and sets the sensitivity of the menu items
 */
void
ruleview_popup_menu (GdkEventButton *bevent)
{
	gtk_menu_popup (GTK_MENU (rules_popup_menu),
	                          NULL, NULL, NULL, NULL,
	                          bevent->button, bevent->time);
}

static void
goto_homepage (void)
{
	gnome_url_show ("http://firestarter.sourceforge.net", NULL);
}

static void
goto_manual (void)
{
	gnome_url_show ("http://firestarter.sourceforge.net/manual", NULL);
}

static gchar *
menu_translate(const char* path, gpointer data)
{
	return gettext (path);
}

static GtkWidget *
menu_create_items (GtkItemFactoryEntry  *entries,
                   guint                 entries_qty,
                   const char           *path,
                   GtkItemFactory      **factory,
                   gpointer              data)
{
	*factory = gtk_item_factory_new (GTK_TYPE_MENU, path, NULL);
	gtk_item_factory_set_translate_func (*factory, menu_translate, NULL, NULL);
	gtk_item_factory_create_items (*factory, entries_qty, entries, data);

	return gtk_item_factory_get_widget (*factory, path);
}

/* [ menu_item_set_checked ]
 * Set the checked state of an CheckMenuItem
 */
static void
menu_item_set_checked (GtkItemFactory *factory,
                       const gchar *path,
		       gboolean active)
{
	GtkCheckMenuItem *item;
	
	g_return_if_fail (factory != NULL);
	
	item = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (factory, path));
	if (item == NULL)
		g_error ("Could not find menu item `%s'", path);
	else
		gtk_check_menu_item_set_active (item, active);
}

/* [ create_hits_popup ]
 * Create the popup menu for the hitview
 */
static GtkWidget *
create_hits_popup (void)
{
	GtkWidget *menu;
	gboolean visible;

	menu = menu_create_items (hits_popup_entries,
	                          hits_popup_entries_qty,
	                          "<HitView>",
	                          &hits_popup_factory,
	                          NULL);

	/* Make the check marks match the column visibility */
	visible = gnome_config_get_bool ("/firestarter/Program/timecol=TRUE");
	menu_item_set_checked (hits_popup_factory, N_("/View columns/Time"), visible);

	visible = gnome_config_get_bool ("/firestarter/Program/incol=FALSE");
	menu_item_set_checked (hits_popup_factory, N_("/View columns/In"), visible);

	visible = gnome_config_get_bool ("/firestarter/Program/outcol=FALSE");
	menu_item_set_checked (hits_popup_factory, N_("/View columns/Out"), visible);
	
	visible = gnome_config_get_bool ("/firestarter/Program/portcol=TRUE");
	menu_item_set_checked (hits_popup_factory, N_("/View columns/Port"), visible);
	
	visible = gnome_config_get_bool ("/firestarter/Program/sourcecol=TRUE");
	menu_item_set_checked (hits_popup_factory, N_("/View columns/Source"), visible);

	visible = gnome_config_get_bool ("/firestarter/Program/destinationcol=FALSE");
	menu_item_set_checked (hits_popup_factory, N_("/View columns/Destination"), visible);

	visible = gnome_config_get_bool ("/firestarter/Program/lengthcol=FALSE");
	menu_item_set_checked (hits_popup_factory, N_("/View columns/Length"), visible);

	visible = gnome_config_get_bool ("/firestarter/Program/toscol=FALSE");
	menu_item_set_checked (hits_popup_factory, N_("/View columns/ToS"), visible);

	visible = gnome_config_get_bool ("/firestarter/Program/protocolcol=TRUE");
	menu_item_set_checked (hits_popup_factory, N_("/View columns/Protocol"), visible);

	visible = gnome_config_get_bool ("/firestarter/Program/servicecol=TRUE");
	menu_item_set_checked (hits_popup_factory, N_("/View columns/Service"), visible);

	return menu;
}

/* [ create_rules_popup ]
 * Create the popup menu for the ruleview
 */
static GtkWidget *
create_rules_popup (void)
{
	GtkWidget *menu;

	menu = menu_create_items (rules_popup_entries,
	                          rules_popup_entries_qty,
	                          "<RuleView>",
	                          &rules_popup_factory,
	                          NULL);

	return menu;
}
