/*---[ util.c ]-------------------------------------------------------
 * Copyright (C) 2000, 2001 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Miscellaneous utility functions
 *--------------------------------------------------------------------*/

#include <sys/stat.h>
#include <stdio.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "globals.h"
#include "firestarter.h"
#include "util.h"

extern int h_errno;

void
error_dialog (gchar *message)
{
	GtkWidget *dialog;

	dialog = gnome_error_dialog_parented (message, GTK_WINDOW (Firestarter.window));
	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
}

/* [ root_alert ]
 * check if the users is root, if not display an alert message
 */
gint
root_alert (void)
{
	gint uid;

	if ((uid = getuid ())) {
		GtkWidget *dialog;

		if (CONSOLE) {
			g_print (_("You must be root to manipulate the firewall.\n"));
			return uid;
		}

		dialog = gnome_error_dialog_parented (
			_("You must be root to manipulate the firewall."),
			GTK_WINDOW (Firestarter.window));
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	}

	return uid;
}

/* [ check_version ]
 * Check version on the script file. If it was created for the wrong
 * kernel type or the version was < program version, start fresh.
 */
void
check_version (void)
{
	FILE *f;
	gchar *version = "0.0.0";
	gchar buf[512];
	gint wrong_kernel = FALSE;

	f = fopen (FIRESTARTER_RULES_DIR "/firestarter/firewall.sh", "r");

	if (f == NULL) {
		gnome_config_set_bool ("/firestarter/Druid/firsttime", TRUE);
		gnome_config_sync ();
		return;
	}

	while (fgets (buf, 512, f) != NULL) {
		if (strstr (buf, "by Firestarter "))  {
			version = get_text_between (buf, "by Firestarter ", ",");
			if (strstr (buf, "NETFILTER")) {
				if (!detect_netfilter())
					wrong_kernel = TRUE;
			} else
				if (detect_netfilter ())
					wrong_kernel = TRUE;
			break;
		}
	}

	fclose (f);

	if ((g_strcasecmp (version, VERSION) < 0) || wrong_kernel) {
		GtkWidget *dialog;
		GtkWidget *label;
		int button;

		mkdir (FIRESTARTER_RULES_DIR "/firestarter", 00700);

		dialog = gnome_dialog_new (_("Old version detected"),
			GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);

		label = gtk_label_new (_(
			"\nA firewall created by an older version of\n"
			"Firestarter has been detected.\n\n"
			"OK to remove old firewall (recommended)?\n"));
		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
			label, FALSE, FALSE, 0);
		gtk_widget_show (label);

		button = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));

		switch (button) {
			case 0:
				remove (FIRESTARTER_RULES_DIR "/firestarter/firewall.sh");
				remove (FIRESTARTER_RULES_DIR "/firestarter/trusted-hosts");
				remove (FIRESTARTER_RULES_DIR "/firestarter/blocked-hosts");
				remove (FIRESTARTER_RULES_DIR "/firestarter/open-ports");
				remove (FIRESTARTER_RULES_DIR "/firestarter/stealthed-ports");
				remove (FIRESTARTER_RULES_DIR "/firestarter/blocked-ports");
				gnome_config_set_bool ("/firestarter/Druid/firsttime", TRUE);
				gnome_config_sync ();
				printf ("Scripts removed");
				break;

			case 1:
			case -1:
				printf ("Scripts not removed");
				break;
		}
	}
}

/* [ detect_netfilter ]
 * Return true if Linux 2.4 or 2.5 kernel found
 */
gboolean
detect_netfilter (void)
{
	FILE *f;
	gchar buf[512];

	/* The detection is only done once */
	static gint detected = 0;

	/* The result of the detection */
	static gboolean result = FALSE;

	if (detected)
		return result;

	f = fopen ("/proc/version", "r");

	/* If we can't determine the kernel version */
	if (f == NULL) {
		perror("/proc/version");
		g_print("Could not determine kernel version,"
			"defaulting to 2.4/iptables\n");
		result = TRUE;
	}
	else
		while (fgets (buf, 512, f) != NULL) {
			if ((strstr (buf, " 2.4.") != NULL) || (strstr (buf, " 2.5.") != NULL)) {
				g_print ("NETFILTER detected\n");
				system ("/sbin/modprobe --autoclean ip_tables");
				result = TRUE;
				fclose (f);
			}
		}

	detected = 1;
	return result;
}

/* [ get_file_path ]
 * Get the correct path to the rule file based on the group number
 */
gchar *
get_file_path (gint rulegroupnum)
{
	gchar *path;

	switch (rulegroupnum) {
	  case RULETYPE_TRUSTED_HOST:
		path = FIRESTARTER_RULES_DIR "/firestarter/trusted-hosts"; break;
	  case RULETYPE_BLOCKED_HOST:
		path = FIRESTARTER_RULES_DIR "/firestarter/blocked-hosts"; break;
	  case RULETYPE_FORWARD:
		path = FIRESTARTER_RULES_DIR "/firestarter/forward"; break;
	  case RULETYPE_OPEN_PORT:
		path = FIRESTARTER_RULES_DIR "/firestarter/open-ports"; break;
	  case RULETYPE_STEALTHED_PORT:
		path = FIRESTARTER_RULES_DIR "/firestarter/stealthed-ports"; break;
	  case RULETYPE_BLOCKED_PORT:
		path = FIRESTARTER_RULES_DIR "/firestarter/blocked-ports"; break;
	  default:
		path = NULL;
	}

	return path;
}

void
print_hit (Hit *h)
{
	printf ("HIT: %s from %s to %s, protocol %s, service %s\n",
		h->time,
		h->source,
		h->destination,
		h->protocol,
		h->service);
}

void
print_rule (Rule *r)
{
	printf ("RULE: type %d, host %s, port %s, active %d\n",
		r->type,
		r->host,
		r->port,
		r->active);
}

Hit *
copy_hit (Hit *h)
{
	Hit *new = g_new (Hit, 1);
	
	new->time = g_strdup (h->time);
	new->in =  g_strdup (h->in);
	new->out = g_strdup (h->out);
	new->port = g_strdup (h->port);
	new->source = g_strdup (h->source);
	new->destination = g_strdup (h->destination);
	new->length = g_strdup (h->length);
	new->tos = g_strdup (h->tos);
	new->protocol = g_strdup (h->protocol);
	new->service = g_strdup (h->service);

	return new;
}

gboolean
hit_is_for_me (Hit *h)
{
	static gchar *myip = NULL;
	
	if (myip == NULL) {
		gchar hostname[40];
		
		gethostname (hostname, 39);
		myip = g_strdup ((gchar *)lookup_ip (hostname));
	}

	return (strcmp (myip, h->destination) == 0);
}

/* [ compare_to_last_hit ]
 * Loosely compare a hit to the previous one.
 * Returns true if they are similiar
 */
gboolean
compare_to_last_hit (Hit *new)
{
	static Hit *old = NULL;
	gboolean same = TRUE;
	
	if (old == NULL) {
		old = copy_hit (new);
		return FALSE;
	}

	/* Going from most likely to differ to least */
	if (strcmp (new->port, old->port) != 0) {
		same = FALSE;
	}
	else if (strcmp (new->protocol, old->protocol) != 0) {
		same = FALSE;
	}
	else if (strcmp (new->source, old->source) != 0) {
		same = FALSE;
	}
	else if (strcmp (new->destination, old->destination) != 0) {
		same = FALSE;
	}
/*	else if (strcmp (new->in, old->in) == 0)
		same = FALSE;
	else if (strcmp (new->out, old->out) == 0)
		same = FALSE;
*/
	g_free (old);

	old = copy_hit (new);

	return same;
}

/* [ create_devicelist ]
 * Create a list of network interfaces
 */
GList*
create_devicelist (void)
{
	GList *devicelist = NULL;
	FILE *f;
	gchar buf[512];
	gchar *name;

	f = fopen ("/proc/net/dev", "r");
	g_assert (f);

	while (fgets (buf, 512, f) != NULL) {
		if (index(buf, ':')) {
			name = g_strndup (buf, index (buf, ':')-buf);
			g_strstrip (name);
			
			if (strcmp (name, "lo")) /* Remove local loopback device from list */
				devicelist = g_list_append (devicelist, name);
		}
	}
	fclose(f);

	return (devicelist);
}

/* [ is_capable_of_nat ]
 * Return true if we can do nat, port forwarding etc.
 */
gboolean
is_capable_of_nat (void)
{
	GList *devices;
	
	devices = (GList *)create_devicelist ();
	return (g_list_length (devices) >= 2);
}

/* [ get_text_between ]
 * Give a string and two subtext markers in the string and the function 
 * returns the text between the markers. Note: Return empty string if fail.
 */
gchar *
get_text_between (const gchar *string, gchar *marker1, gchar *marker2)
{
	gint i = strlen (marker1);

	marker1 = strstr (string, marker1);
	if (marker1 == NULL)
		return "";

	marker1 += i;

	marker2 = strstr (marker1, marker2);
	if (marker2 == NULL)
		return "";

	return g_strndup (marker1, marker2-marker1);
}

/* [ lookup_ip ]
 * Resolve an IP address given in dotted-decimal notation into
 * an hostname or vice versa.
 */
gchar *
lookup_ip (gchar *ip)
{
	struct hostent *hostentry = NULL;
	struct in_addr address;

	if (inet_aton (ip, &address)) {
		hostentry = gethostbyaddr ((char *)&address,
					   sizeof (address), AF_INET);
		if (hostentry != NULL)
			return hostentry->h_name;
		else
			return ip;
	} else {
		hostentry = gethostbyname (ip);
		memcpy (&address.s_addr, hostentry ->h_addr, hostentry->h_length);

		return inet_ntoa(address);
	}
}

/* [ is_a_valid_port ]
 * Test that port is a valid number
 */
gboolean
is_a_valid_port (const gchar *port)
{
	int port_num;

	/* TODO: range detection */
	port_num = atoi (port);
	return (port_num >= 1 && port_num <= 65535);
}

/* [ is_a_valid_host ]
 * _Very_ loose host string checking
 */
gboolean
is_a_valid_host (const gchar *host)
{
	static GPatternSpec *pattern = NULL;
	gboolean valid = FALSE;
	gint length;

	if (pattern == NULL)
		pattern = g_pattern_spec_new ("*.*");

	length = strlen (host);

	/* Host name must contain a dot and be over 3 but less than 255 chars long */
	if (length > 3 && length < 255)
		if (g_pattern_match_string (pattern, host))
			valid = TRUE;

	return valid;
}
